<?php
/*
 * @package Joomla 2.5
 * @copyright Copyright (C) 2005 Open Source Matters. All rights reserved.
 * @license GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html, see LICENSE.php
 *
 * @module OpenSim Gridstatus
 * @copyright Copyright (C) 2012 FoTo50 http://www.jopensim.com
 * @license GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html
 */

//no direct access
defined('_JEXEC') or die('Direct Access to this location is not allowed.');

class ModOpenSimHelper {

	public $offlinecolor = "#ff0000";
	public $onlinecolor = "#00ff00";
	public $opensim;
	public $_osgrid_db;
	public $connectparams = array();
	public $parameter = array();

	public function __construct($connect,$parameter) {
		$this->connectparams['osgriddbhost']	= null; // initialise to avoid php notices
		$this->connectparams['osgriddbuser']	= null;
		$this->connectparams['osgriddbpasswd']	= null;
		$this->connectparams['osgriddbname']	= null;
		$this->connectparams['osgriddbport']	= null;
		$this->parameter['gridboxlines']		= null;
		$this->parameter['gridlines']			= null;
		$this->parameter['lastonline']			= null;
		$this->parameter['gridstatus']			= null;
		$this->parameter['totalregions']		= null;
		$this->parameter['lastDays']			= null;
		$this->parameter['hiddenregions']		= 0;
		if(array_key_exists("offlinecolor",$parameter)	&& $parameter['offlinecolor'])	$this->offlinecolor					= $parameter['offlinecolor'];
		if(array_key_exists("onlinecolor",$parameter)	&& $parameter['onlinecolor'])	$this->onlinecolor					= $parameter['onlinecolor'];
		if(array_key_exists("gridlines",$parameter)		&& $parameter['gridlines'])		$this->parameter['gridlines']		= $parameter['onlinecolor'];
		if(array_key_exists("lastonline",$parameter)	&& $parameter['lastonline'])	$this->parameter['lastonline']		= $parameter['lastonline'];
		if(array_key_exists("gridstatus",$parameter)	&& $parameter['gridstatus'])	$this->parameter['gridstatus']		= $parameter['gridstatus'];
		if(array_key_exists("totalregions",$parameter)	&& $parameter['totalregions'])	$this->parameter['totalregions']	= $parameter['totalregions'];
		if(array_key_exists("lastDays",$parameter)		&& $parameter['lastDays'])		$this->parameter['lastDays']		= $parameter['lastDays'];
		if(array_key_exists("hiddenregions",$parameter)	&& $parameter['hiddenregions'])	$this->parameter['hiddenregions']	= $parameter['hiddenregions'];
		
		if(_OPENSIMCOMPONENT) {
			$this->getComponentParameter();
			$this->getComponentSettings($parameter);
		} else {
			if(is_array($connect)) {
				foreach($connect AS $key => $val) {
					$this->connectparams[$key] = $val;
				}
			}
			$this->setParameter($parameter);
		}

		$this->initOpenSim();								// generate the opensim object
		$this->_osgrid_db = $this->getOsGridDB();			// load the external DB in an object
	}

	public function initOpenSim() {
		$this->opensim = new opensim(null,null,null,null,null,$this->connectparams['osgriddbhost'],$this->connectparams['osgriddbuser'],$this->connectparams['osgriddbpasswd'],$this->connectparams['osgriddbname'],$this->connectparams['osgriddbport']);
	}

	public function getComponentParameter() {
		$params = &JComponentHelper::getParams('com_opensim');
		$parameter['osgriddbhost']   = ($params->get('opensimgrid_dbhost'))   ? $params->get('opensimgrid_dbhost'):$params->get('opensim_dbhost');
		$parameter['osgriddbuser']   = ($params->get('opensimgrid_dbuser'))   ? $params->get('opensimgrid_dbuser'):$params->get('opensim_dbuser');
		$parameter['osgriddbpasswd'] = ($params->get('opensimgrid_dbpasswd')) ? $params->get('opensimgrid_dbpasswd'):$params->get('opensim_dbpasswd');
		$parameter['osgriddbname']   = ($params->get('opensimgrid_dbname'))   ? $params->get('opensimgrid_dbname'):$params->get('opensim_dbname');
		$parameter['osgriddbport']   = ($params->get('opensimgrid_dbport'))   ? $params->get('opensimgrid_dbport'):$params->get('opensim_dbport');
		$this->connectparams = $parameter;
	}

	public function _buildSettingsQuery() {
		$query = "SELECT #__opensim_config.* FROM #__opensim_config";
		return $query;
	}

	public function getComponentSettings($parameter) {
		$db =& JFactory::getDBO();
		$query = $this->_buildSettingsQuery();
		$db->setQuery($query);
		$jOpenSimSettings = $db->loadAssocList();
		if(is_array($jOpenSimSettings) && count($jOpenSimSettings) > 0) {
			$settings = array();
			foreach($jOpenSimSettings AS $jOpenSimSetting) {
				if($jOpenSimSetting['configname'] == "lastnamelist") {
					$lastnames = explode("\n",$jOpenSimSetting['configvalue']);
					if(is_array($lastnames) && count($lastnames) > 0) {
						foreach($lastnames AS $lastname) {
							if(trim($lastname)) $settings['lastnames'][] = trim($lastname);
						}
					}
					$settings['lastnamelist'] = $jOpenSimSetting['configvalue'];
					continue;
				}
				switch($jOpenSimSetting['configtype']) {
					case "int":
						$settings[$jOpenSimSetting['configname']] = intval($jOpenSimSetting['configvalue']);
					break;
					case "float":
						$settings[$jOpenSimSetting['configname']] = floatval($jOpenSimSetting['configvalue']);
					break;
					default:
						$settings[$jOpenSimSetting['configname']] = strval($jOpenSimSetting['configvalue']);
					break;
				}
				if($jOpenSimSetting['configname'] == "loginscreen_xdays") {
					if($settings[$jOpenSimSetting['configname']] > 0) $this->parameter['lastDays']	= $settings[$jOpenSimSetting['configname']];
				} else {
					$this->parameter[$jOpenSimSetting['configname']]	= $settings[$jOpenSimSetting['configname']];
				}
			}
		} else {
			return FALSE;
		}
		$this->parameter['offlinecolor']	= $this->offlinecolor;
		$this->parameter['onlinecolor']		= $this->onlinecolor;
	}

	public function setParameter($parameter) {
		if(!is_array($parameter)) return FALSE;
		$this->parameter['lastDays']		= $parameter['lastDays'];
		$this->parameter['offlinecolor']	= $parameter['offlinecolor'];
		$this->parameter['onlinecolor']		= $parameter['onlinecolor'];
		$this->parameter['gridstatus']		= $parameter['gridstatus'];
		$gridlines = 0;
		if($parameter['showgridstatus']) $gridlines += 1;
		if($parameter['showregions']) $gridlines += 2;
		if($parameter['showlastvisitors']) $gridlines += 4;
		if($parameter['showonline']) $gridlines += 8;
		$this->parameter['gridlines'] = $gridlines;
	}

	public function getOsGridDB() {
		return $this->opensim->_osgrid_db;
	}

	public function getStatus() {
		if(!$this->_osgrid_db) {
			$returnvalue['statusmsg'] = "<font color='".$this->offlinecolor."'>".JText::_('MOD_GRIDSTATUS_OFFLINE')."</font>";
		} else {
			$zeroUID		= "00000000-0000-0000-0000-000000000000";
			$lastDays		= $this->parameter['lastDays'];
			$offlinecolor	= $this->offlinecolor;
			$onlinecolor	= $this->onlinecolor;
			$returnvalue = array();

			$this->_osgrid_db->setQuery("SELECT uuid FROM regions");
			$regions = $this->_osgrid_db->loadResultArray();
			if($this->parameter['hiddenregions'] == 0) {
				$db = JFactory::getDbo();
				$query = "SELECT #__opensim_mapinfo.regionUUID FROM #__opensim_mapinfo WHERE #__opensim_mapinfo.hidemap = 1";
				$db->setQuery($query);
				$db->query();
				$numrows = $db->getNumRows();
				if($numrows > 0) {
					$hiddenregions = $db->loadResultArray();
					foreach($hiddenregions AS $hiddenregion) {
						$ishidden = array_search($hiddenregion,$regions);
						if($ishidden === FALSE) continue;
						else unset($regions[$ishidden]);
					}
					$returnvalue['totalregions'] = var_export($regions,TRUE);
				}
			}
			$returnvalue['totalregions'] = count($regions);


			if($this->parameter['gridstatus'] == -1) $returnvalue['status'] = "offline";
			elseif($this->parameter['gridstatus'] == 1) $returnvalue['status'] = "online";
			else {
				if($returnvalue['totalregions'] > 0)  $returnvalue['status'] = "online"; // Online Server needs more than 0 regions
				else  $returnvalue['status'] = "offline";
			}

			if($returnvalue['status'] == "online") $returnvalue['statusmsg'] = "<font color='".$this->onlinecolor."'>".JText::_('MOD_GRIDSTATUS_ONLINE')."</font>";
			else $returnvalue['statusmsg'] = "<font color='".$this->offlinecolor."'>".JText::_('MOD_GRIDSTATUS_OFFLINE')."</font>";

			$returnvalue['days'] = $lastDays;

			$this->_osgrid_db->setQuery(sprintf("SELECT COUNT(*) FROM Presence WHERE RegionID != '%s'",$zeroUID));
			$returnvalue['online'] = $this->_osgrid_db->loadResult();

			$tage = sprintf("%d",$lastDays);
			$jetzt = time();
			$lastloggedin = $jetzt - 60*60*24*$tage;
			$this->_osgrid_db->setQuery("SELECT COUNT(*) FROM GridUser WHERE Login > '$lastloggedin' OR Logout > '$lastloggedin'");
			$returnvalue['lastonline'] = $this->_osgrid_db->loadResult();

			$returnvalue['gridboxlines'] = $this->parameter['gridlines'];
		}
		return $returnvalue;
	} //end getStatus

} //end ModOpenSimHelper
?>
